@@ -1,5 +1,8 @@
 """ Contains construct adapters and structures. """
 from construct import Struct, Adapter, ExprAdapter, Int8ub, Enum, FlagsEnum, Const, Pass, GreedyRange, GreedyBytes, IfThenElse, If, Bytes, Byte
+import struct
+from datetime import datetime, time
+from math import floor
 
 PROP_INFO_RETURN = 2
 PROP_SCHEDULE_RETURN = 0x21
@@ -7,6 +10,7 @@ PROP_SCHEDULE_RETURN = 0x21
 NAME_TO_DAY = {"sat": 0, "sun": 1, "mon": 2, "tue": 3, "wed": 4, "thu": 5, "fri": 6}
 HOUR_24_PLACEHOLDER=1234
 
+
 class TimeAdapter(Adapter):
     """ Adapter to encode and decode schedule times. """
     def _decode(self, obj, ctx):
@@ -20,13 +24,14 @@ class TimeAdapter(Adapter):
         if obj == HOUR_24_PLACEHOLDER:
             return 24 * 60 / 10
 
-        encoded = (obj.hour * 60 + obj.minute) / 10
+        encoded = int((obj.hour * 60 + obj.minute) / 10)
+
         return encoded
 
 class TempAdapter(Adapter):
     """ Adapter to encode and decode temperature. """
     def _decode(self, obj, ctx):
-        return obj / 2.0
+        return float(obj / 2.0)
 
     def _encode(self, obj, ctx):
         return int(obj * 2.0)
@@ -42,6 +47,7 @@ ModeFlags = "ModeFlags" / FlagsEnum(Int8ub,
                                     UNKNOWN=0x40,
                                     LOW_BATTERY=0x80)
 
+
 class AwayDataAdapter(Adapter):
     """ Adapter to encode and decode away data. """
     def _decode(self, obj, ctx):
@@ -56,10 +62,12 @@ class AwayDataAdapter(Adapter):
         return datetime(year=year, month=month, day=day, hour=hour, minute=min)
 
     def _encode(self, obj, ctx):
+        if obj.year < 2000 or obj.year > 2099:
+            raise Exception("Invalid year, possible [2000,2099]")
         year = obj.year - 2000
         hour = obj.hour * 2
         if obj.minute:  # we encode all minute values to h:30
-            hour = hour | 0x01
+            hour |= 0x01
 
         value = struct.pack("BBBB", obj.day, year, hour, obj.month)
         return value
